
// EchoLClient.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, September 2005

/* Open a L2CAP (packet-based) link to the server.

   Communication is triggered by echoMessage() being called,
   which sends a message to the server, and waits for an
   answer. 

   closeDown() closes the server link.

   EchoLClient uses the same readData() and sendMessage()
   methods as ThreadedEchoLHandler.

   ----------------------
   EchoLClient is based on EchoClient in EchoClientMIDlet, but has
   been changed to deal with L2CAP communication rather than RFCOMM.

      * conn is an instance of L2CAPConnection
      * removed IO streams
      * get negotiated transmit and receive MTUs
      * rewrote sendMessage() to use send()
      * rewrote readData() to use read()
*/


import javax.bluetooth.*;
import javax.microedition.io.*;
import java.io.*;
 

public class EchoLClient extends Thread
{
  private ServiceRecord servRecord;
  private ClientForm clientForm;

  private L2CAPConnection conn;  // for the server

  private boolean isClosed = true;   
       // is the connection to the server closed?

  // actual sending and receiving packet size
  private int transmitMTU, receiveMTU;


  public EchoLClient(ServiceRecord sr, ClientForm cf) 
  {
    clientForm = cf;
    servRecord = sr;
  }  // end of EchoLClient()


  public void run()
  /* The server connection is set up in this thread since
     the call to Connector.open() may block for a long period,
     and we don't want that to affect the ClientForm GUI.
  */
  {
    // get a URL for the service
    String servURL = servRecord.getConnectionURL(
                            ServiceRecord.NOAUTHENTICATE_NOENCRYPT, false);
    System.out.println("Service URL: " + servURL);

    if (servURL != null) {
      clientForm.setStatus("Found Echo Server URL");
      try {
        // connect to the server, and extract IO streams
        conn = (L2CAPConnection) Connector.open(servURL);
        // System.out.println("Opened a connection to server");

        // get negotiated sending and receiving MTUs
        transmitMTU = conn.getTransmitMTU();
        receiveMTU = conn.getReceiveMTU();
        System.out.println("transmitMTU: " + transmitMTU +
                         "; receiveMTU: " + receiveMTU);

        clientForm.setStatus("Connected to Echo Server");
        clientForm.setEnable(true);   // communication allowed
        isClosed = false;   // i.e. the connection is open
      } 
      catch (Exception ex) 
      {  System.out.println(ex);  
         clientForm.setStatus("Connection Failed");
      } 
    }
    else
      clientForm.setStatus("No Service Found");
  }  // end of run()



  public void closeDown()
  // send "bye$$", then close the link with the service
  {
    if (!isClosed) {
      sendMessage("bye$$");  // tell the server that the client is leaving
      try {
        if (conn != null)
          conn.close();
      }
      catch (IOException e) 
      {  System.out.println(e);  }
      isClosed = true;
    }
  }  // end of closeDown();


  // --------------------- IO methods ------------------
  // readData() and sendMessage are the same as the methods 
  // in ThreadedEchoLHandler


  public String echoMessage(String msg)
  /* This method is used by ClientForm to send a message (msg) to 
     the server, and wait for an answer. The response is returned. 

     If there's a problem, the client is notified using disableClient(),
     and null is returned from this method. 
  */
  {
    if (isClosed) {
      disableClient("No Connection to Server");
      return null;
    }

    if ((msg == null) || (msg.trim().equals("")))
      return "Empty input message";
    else {
      if (sendMessage(msg)) {    // message sent ok
        String response = readData();  // wait for response
        if (response == null) {
          disableClient("Server Terminated Link");
          return null;
        }
        else  // there was a response
          return response;
      }
      else {   // unable to send message
        disableClient("Connection Lost");
        return null;
      }
    }
  }  // end of echoMessage()


  private void disableClient(String msg)
  /* Notify the client of the communications error, 
     and prevent any further communication. */
  { clientForm.setStatus(msg);
    clientForm.setEnable(false);
  }  // end of disableClient()


  private boolean sendMessage(String msg)
  // send the message (if it's not too long)
  {
    byte[] msgBytes = msg.getBytes();
    if (msgBytes.length > transmitMTU) {
      System.out.println("Message too long, sending \"??\" instead");
      msgBytes = "??".getBytes();
    }
    try {
      conn.send(msgBytes);
      return true;
    } 
    catch (Exception e) 
    {  System.out.println("sendMessage(): " + e);  
       return false;
    } 
  }  // end of sendMessage()


  private String readData()
  // read a message
  { 
    byte[] data = new byte[receiveMTU];
    try {
      int len = conn.receive(data);
      String msg = new String(data, 0, len);
      return msg;
    } 
    catch (Exception e) 
    {  System.out.println("readData(): " + e); 
       return null;
    } 
  }   // end of readData()

} // end of EchoLClient.java