
// ClientInfo.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, September 2005

/* Store various information about the client's connection
   to the server. The methods allow messages to be read and sent,
   and for the connection to be closed.
*/

import java.io.*;
import javax.bluetooth.*;


public class ClientInfo
{
  private L2CAPConnection conn; // client connection
  private int transmitMTU, receiveMTU;    // packet sizes
  private String clientName;


  public ClientInfo(L2CAPConnection c)
  {
    conn = c;
    System.out.println("Client Info");
    try {
      // get negotiated sending and receiving MTUs
      transmitMTU = conn.getTransmitMTU();
      receiveMTU = conn.getReceiveMTU();
      System.out.println("transmitMTU: " + transmitMTU +
                       "; receiveMTU: " + receiveMTU);

      // store the name of the connected client
      RemoteDevice rd = RemoteDevice.getRemoteDevice(conn);
      clientName = getDeviceName(rd);
      System.out.println("Client name: " + clientName);
    }
    catch(Exception e) 
    { System.out.println(e); }
  }  // end of ClientInfo()


  private String getDeviceName(RemoteDevice dev)
  /* Return the 'friendly' name of the device being examined,
     or "Device ??" */
  {
    String devName;
    try {
      devName = dev.getFriendlyName(false);  // to reduce connections
    }
    catch (IOException e) 
    { devName = "Device ??";  }
    return devName;
  }  // end of getDeviceName()


  public boolean isReady()
  { try {
      return conn.ready();
    }  
    catch (IOException e) 
    {  return false;  }
  } // end of isReady()


  public void closeDown()
  // close down the handler
  {
    System.out.println("Closed echo handler");
    try {
      if (conn != null)
        conn.close();
    }
    catch (IOException e) 
    {  System.out.println(e);  }
  }  // end of closeDown()


  public String getClientName()
  {  return clientName;  }


  // --------------- IO methods ---------------------------
  /* readData() and sendMessage() are the same as the methods 
     in ThreadedEchoLHandler */


  public String readData()
  // read a message, no bigger than receiveMTU
  { 
    byte[] data = new byte[receiveMTU];
    try {
      int len = conn.receive(data);
      String msg = new String(data, 0, len);
      return msg;
    } 
    catch (Exception e) 
    {  System.out.println("readData(): " + e); 
       return null;
    } 
  }   // end of readData()


  public boolean sendMessage(String msg)
  // send the message (if it's not too big)
  {
    byte[] msgBytes = msg.getBytes();
    if (msgBytes.length > transmitMTU) {
      System.out.println("Message too long, sending \"??\" instead");
      msgBytes = "??".getBytes();
    }
    try {
      conn.send(msgBytes);
      return true;
    } 
    catch (Exception e) 
    {  System.out.println("sendMessage(): " + e);  
       return false;
    } 
  }  // end of sendMessage()


} // end of ClientInfo()