
// ConnectionsHandler.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, September 2005

/* A threaded handler, called by EchoLPServer to deal with all
   the clients connections.

   The loop in the run() method repeatedly cycles through all the 
   clients, checking if a client has a message waiting to be processed
   When a message comes in, it is sent back converted to uppercase.

   The handler also communicates with the top-level MIDlet (ecm)
   to add and decrement the count of clients, and to display a 
   received message.

   closeDown() terminates the handler.

   Client details are stored in ClientInfo objects, which are 
   collected together in a clients Vector instance.
   ClientInfo contains the methods for doing client IO.
*/

import javax.microedition.io.*;
import javax.bluetooth.*;
import java.util.*;


public class ConnectionsHandler extends Thread
{
  private static final int DELAY = 1000;   // ms delay (1 sec)

  private EchoLPServerMIDlet ecm;  // top-level MIDlet
  private Vector clients;   
       // vector of ClientInfo objects, one for each client
  private boolean isRunning;


  public ConnectionsHandler(EchoLPServerMIDlet ecm)
  {
    this.ecm = ecm;
    clients = new Vector();
    isRunning = true;
  } // end of ConnectionsHandler()


  public void addConnection(L2CAPConnection conn)
  // store a new ClientInfo object holding the client connection
  {
    ecm.incrCount();   // tell top-level MIDlet there's a new client
    ClientInfo ci = new ClientInfo(conn);  // create client info
    clients.addElement(ci);
    System.out.println("Added client " + (clients.size()-1));
  }  // end of addConnection()


  public void run()
  /* Iterate through the client connections stored in the clients
     vector, examining each one to see if there's a message.
     Process the message, and close the client connection if
     the client has finished. 

     If there's been no activity after examining all the clients, 
     then sleep a while before resuming the polling.
  */
  {
    boolean foundMessage = false;
    ClientInfo ci;
    int i = 0;   // index into the clients vector

    while (isRunning) {
      waitWhileEmpty();
      ci = (ClientInfo) clients.elementAt(i);

      if (ci.isReady()) {    // ci has a message waiting to be processed
        foundMessage = true;
        boolean isCliActive = processClient(ci);
        if (!isCliActive) {   // the client connection should be closed
          System.out.println("Closing client connection " + i);
          ci.closeDown();
          clients.removeElementAt(i);   // remove from the vector
          ecm.decrCount();      // remove from the MIDlet's count
        }
        else   // this client is active
          i++;   // move to next client
      }
      else  // this client didn't have a message ready for processing
        i++;   // move to next client

      if (i == clients.size()) {   // at the end of the vector
        if (!foundMessage) {   // no clients sent msgs during the loop
          try {
            // System.out.println("waiting since no activity...");
            Thread.sleep(DELAY);      // wait a while
          } 
          catch (Exception ex) {}
        }
        i = 0;   // start examining the clients again
        foundMessage = false;
      }
    }
  }  // end of run()


  private void waitWhileEmpty()
  // wait until there's some client connections
  {
    while (clients.size() == 0) {  // no clients
      try {
        // System.out.println("waiting since no clients...");
        Thread.sleep(DELAY);      // wait a while
      } 
      catch (Exception ex) {}
    }
  }  // end of waitWhileEmpty()


  private boolean processClient(ClientInfo ci)
  /* When a client message comes in, echo it back in uppercase.

     If the client sends "bye$$", or there's a problem, then
     return false to signal that the client connection should be 
     closed.
  */
  {
    boolean isCliActive = true;
    String clientName = ci.getClientName();

    String line;
    if((line = ci.readData()) == null)
      isCliActive = false;
    else {  // there was some input
      System.out.println(clientName + " sent: " + line);
      if (line.trim().equals("bye$$"))
        isCliActive = false;
      else {
        ecm.showMessage(clientName + ": " + line);  // show at top-level
        String upper = line.trim().toUpperCase();
        if (isRunning)
          ci.sendMessage(upper);
      }
    }
    return isCliActive;
  }  // end of processClient()


  public void closeDown()
  // close all the clients connections
  {
    ClientInfo ci;
    isRunning = false;  // to stop run() loop
    System.out.println("Closing all clients connections");
    for (int i=0; i < clients.size(); i++) {
      ci = (ClientInfo) clients.elementAt(i);
      ci.closeDown();
    }
    clients.removeAllElements();   // empty the vector
  }  // end of closeDown()

}  // end of ConnectionsHandler class