
// EchoLPServer.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, September 2005

/* EchoLPService offers a L2CAP service with a specified 
   UUID (stored in UUID_STRING) and name (in SERVICE_NAME). 

   The waiting for client connections is done in a
   thread so that it doesn't block the top-level MIDlet.

   As a client connects, the connection is passed to a
   ConnectionsHandler thread. It uses polling to periodically
   check all the client connections, and to deal with messages.

   This approach means that the server-side only uses two threads
   (one for EchoLPServer, one for ConnectionsHandler), unlike
   EchoLPService which employs one handler thread for each client.

   The server and handler are terminated by calling closeDown().

   ---------------
   Based on EchoLServer in EchoLServerMIDlet.
*/

import java.io.*;
import javax.microedition.io.*;
import javax.bluetooth.*;


public class EchoLPServer extends Thread 
{
  // UUID and name of the echo service
  private static final String UUID_STRING = "22222222222222222222222222222222";
       // 32 hex digits which will become a 128 bit ID
  private static final String SERVICE_NAME = "echolserver";   // use lowercase

  private EchoLPServerMIDlet ecm;
  private L2CAPConnectionNotifier server;

  private ConnectionsHandler handler;
     /* handler will poll all the client connections, and respond to 
        their messages */   

  private boolean isRunning = false;


  public EchoLPServer(EchoLPServerMIDlet ecm)
  // Create a server connection
  {
    this.ecm = ecm;

    handler = new ConnectionsHandler(ecm);  // create connections handler
    handler.start();

    try {  // make the server's device discoverable
      LocalDevice local = LocalDevice.getLocalDevice();
      local.setDiscoverable(DiscoveryAgent.GIAC);
    }
    catch (BluetoothStateException e) {
      System.out.println(e);
      return;
    }

    /* Create a L2CAP connection notifier for the server, with 
       the given UUID and name. This also creates a service record. */
    try {  
      server = (L2CAPConnectionNotifier) Connector.open(
          "btl2cap://localhost:" + UUID_STRING + ";name=" + SERVICE_NAME);
    }
    catch (IOException e) {
      System.out.println(e);
      return;
    }
  }  // end of EchoLPServer()


  public void run()
  // Wait for client connections, creating a handler for each one
  {
    isRunning = true;
    try {
      while (isRunning) {
        L2CAPConnection conn = server.acceptAndOpen(); 
           // wait for a client connection
           /* acceptAndOpen() also adds the service record to the 
              device's SDDB, making the service visible to clients */
        handler.addConnection(conn);  // pass the connection to the handler
      }
    }
    catch (IOException e) 
    {  System.out.println(e);  }
  }  // run()


  public void closeDown()
  /* Stop accepting any further client connections, and close down
     the handler. */
  {
    System.out.println("Close down server");
    if (isRunning) {
      isRunning = false;
      try {
        server.close();  
           // close connection, and remove service record from SDDB
      }
      catch (IOException e) 
      {  System.out.println(e);  }

      handler.closeDown();      // close down the handler
    }
  }  // end of closeDown();


} // end of EchoLPServer class
