
// EchoLServer.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, September 2005

/* EchoLPService offers a L2CAP service with a specified 
   UUID (stored in UUID_STRING) and name (in SERVICE_NAME).

   The waiting for client connections is done in a
   thread so that it doesn't block the top-level MIDlet.

   As a client connects, a ThreadedEchoLHandler thread is 
   spawned to deal with the client. This means there will be
   one thread for every client connection. This can be compared
   with the L2CAP polling approach, implemented in EchoLPServer,
   which only creates a single thread to manage all the 
   connetions.

   The server and handlers are terminated by calling closeDown().

   ---------------
   Based on EchoServer in EchoLServerMIDlet
     - changed values for UUID_STRING and SERVICE_NAME
     - StreamConnectionNotifier --> L2CAPConnectionNotifier
     - btspp URL --> btl2cap URL
     - StreamConnection --> L2CAPConnection

*/

import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.bluetooth.*;


public class EchoLServer extends Thread 
{
  // UUID and name of the echo service
  private static final String UUID_STRING = "22222222222222222222222222222222";
       // 32 hex digits which will become a 128 bit ID
  private static final String SERVICE_NAME = "echolserver";   // use lowercase

  private EchoLServerMIDlet ecm;
  private L2CAPConnectionNotifier server;

  private Vector handlers;  // stores the ThreadedEchoLHandler objects
  private boolean isRunning = false;


  public EchoLServer(EchoLServerMIDlet ecm)
  // Create a server connection
  {
    this.ecm = ecm;
    handlers = new Vector();
    try {  // make the server's device discoverable
      LocalDevice local = LocalDevice.getLocalDevice();
      local.setDiscoverable(DiscoveryAgent.GIAC);
    }
    catch (BluetoothStateException e) {
      System.out.println(e);
      return;
    }

    /* Create a L2CAP connection notifier for the server, with 
       the given UUID and name. This also creates a service record. */
    try {  
      server = (L2CAPConnectionNotifier) Connector.open(
          "btl2cap://localhost:" + UUID_STRING + ";name=" + SERVICE_NAME);
    }
    catch (IOException e) {
      System.out.println(e);
      return;
    }
  }  // end of EchoLServer()


  public void run()
  // Wait for client connections, creating a handler for each one
  {
    isRunning = true;
    try {
      while (isRunning) {
        L2CAPConnection conn = server.acceptAndOpen(); 
           // wait for a client connection
           /* acceptAndOpen() also adds the service record to the 
              device's SDDB, making the service visible to clients */

        ThreadedEchoLHandler hand = new ThreadedEchoLHandler(conn, ecm);  
              // create client handler
        handlers.addElement(hand);
        hand.start();
      }
    }
    catch (IOException e) 
    {  System.out.println(e);  }
  }  // end of run()


  public void closeDown()
  /* Stop accepting any further client connections, and close down
     all the handlers. */
  {
    System.out.println("Close down server");
    if (isRunning) {
      isRunning = false;
      try {
        server.close();  
           // close connection, and remove service record from SDDB
      }
      catch (IOException e) 
      {  System.out.println(e);  }

      // close down the handlers
      ThreadedEchoLHandler hand;
      for (int i=0; i < handlers.size(); i++) {
         hand = (ThreadedEchoLHandler) handlers.elementAt(i);
         hand.closeDown();
      }
      handlers.removeAllElements();
    }
  }  // end of closeDown();


} // end of EchoLServer class
