
// ThreadedEchoLHandler.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, September 2005

/* A threaded handler, called by EchoLServer to deal with a client.

   When a message comes in, it is sent back converted to uppercase.

   The handler also communicates with the top-level MIDlet (ecm)
   to add and decrement itself from the count of handlers, and
   to display a received message.

   closeDown() terminates the handler.

   ThreadedEchoLHandler uses the same readData() and sendMessage()
   methods as EchoClient.

   --------------------
   Based on ThreadedEchoHandler, but...
     - RFCOMM --> L2CAP 
     - StreamConnection --> L2CAPConnection
     - removed IO streams
     - get negotiated transmit and receive MTUs
     - readData() and sendMessage() use read() and send()
       in the same way as those methods in EchoLClient

     - merged processClient() into run()
*/

import java.io.*;
import javax.microedition.io.*;
import javax.bluetooth.*;


public class ThreadedEchoLHandler extends Thread
{
  private EchoLServerMIDlet ecm;  // top-level MIDlet

  private L2CAPConnection conn; // client connection

  // actual sending and receiving packet size
  private int transmitMTU, receiveMTU;

  private boolean isRunning = false;
  private String clientName;


  public ThreadedEchoLHandler(L2CAPConnection conn, EchoLServerMIDlet ecm)
  {
    this.conn = conn;
    this.ecm = ecm;

    System.out.println("Client Handler spawned");
    try {
      // get negotiated sending and receiving MTUs
      transmitMTU = conn.getTransmitMTU();
      receiveMTU = conn.getReceiveMTU();
      System.out.println("transmitMTU: " + transmitMTU +
                       "; receiveMTU: " + receiveMTU);

      // store the name of the connected client
      RemoteDevice rd = RemoteDevice.getRemoteDevice(conn);
      clientName = getDeviceName(rd);
      System.out.println("Client name: " + clientName);
    }
    catch(Exception e)
    { System.out.println(e); }
  } // end of ThreadedEchoLHandler()


  private String getDeviceName(RemoteDevice dev)
  /* Return the 'friendly' name of the device being examined,
     or "Device ??" */
  {
    String devName;
    try {
      devName = dev.getFriendlyName(false);  // to reduce connections
    }
    catch (IOException e) 
    { devName = "Device ??";  }
    return devName;
  }  // end of getDeviceName()


  public void run()
   /* When a client message comes in, echo it back in uppercase.
      If the client sends "bye$$", or there's a problem, then
      terminate processing, and the handler.
   */
  {
    ecm.incrCount();   // tell top-level MIDlet there's a new handler
    isRunning = true;
    String line;
    while (isRunning) {
      if((line = readData()) == null)
        isRunning = false;
      else {  // there was some input
        System.out.println(clientName + " sent: " + line);
        if (line.trim().equals("bye$$"))
          isRunning = false;
        else {
          ecm.showMessage(clientName + ": " + line);  // show at top-level
          String upper = line.trim().toUpperCase();
          if (isRunning)
            sendMessage(upper);
        }
      }
    }
    System.out.println("Handler finished");
    ecm.decrCount();  // remove this handler from the top-level count
  }  // end of run()


  public void closeDown()
  // close down the handler
  {
    System.out.println("Closing down echo handler");
    isRunning = false;   // cause run() loop to finish
    try {
      if (conn != null)
        conn.close();
    }
    catch (IOException e) 
    {  System.out.println(e);  }
  }  // end of closeDown()


  // --------------- IO methods ---------------------------
  // Same as the methods in EchoLClient


  private String readData()
  // read a message, no bigger than receiveMTU
  { 
    byte[] data = new byte[receiveMTU];
    try {
      int len = conn.receive(data);
      String msg = new String(data, 0, len);
      return msg;
    } 
    catch (Exception e) 
    {  System.out.println("readData(): " + e); 
       return null;
    } 
  }   // end of readData()


  private boolean sendMessage(String msg)
  // send the message (if it's not too big)
  {
    byte[] msgBytes = msg.getBytes();
    if (msgBytes.length > transmitMTU) {
      System.out.println("Message too long, sending \"??\" instead");
      msgBytes = "??".getBytes();
    }
    try {
      conn.send(msgBytes);
      return true;
    } 
    catch (Exception e) 
    {  System.out.println("sendMessage(): " + e);  
       return false;
    } 
  }  // end of sendMessage()


}  // end of ThreadedEchoLHandler class